/*
 * Copyright © 2016 Quentin "Sardem FF7" Glidic
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the next
 * paragraph) shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
/*
 * (C) Copyright 2014, Fuji Xerox Co.,Ltd.  All rights reserved.
 * File name : fx-seat.h
 * Title : shell seat
 */

#ifndef _FX_SEAT_H_
#define _FX_SEAT_H_

#include "compositor.h"
#include "weston-fx-shell.h"

#ifdef __cplusplus
extern "C" {
#endif

/* Typedefs */
struct fx_shell_seat;

/* Procedures */
struct fx_shell_seat *
    fx_shell_seat_from_seat( struct weston_seat *seat );
bool
    fx_shell_seat_popup_grab_start(
            struct fx_shell_seat *shell_seat,
            struct wl_client *client,
            uint32_t serial );
void
    fx_shell_seat_popup_grab_add_surface(
            struct fx_shell_seat *shell_seat,
            struct wl_list *link );
void
    fx_shell_seat_popup_grab_remove_surface(
            struct fx_shell_seat *shell_seat,
            struct wl_list *link );
void
    fx_shell_seat_set_disabled(
            struct fx_shell_seat *shell_seat,
            bool disabled );
bool
    fx_shell_seat_disabled(
            struct fx_shell_seat *shell_seat );

#ifdef __cplusplus
}
#endif

#endif /* _FX_SEAT_H_ */

